'************************************************************
'*  File	: L3301.BAS                                      
'* Compiler : PICBASIC PRO V2.44 or higher (demo not support)
'************************************************************
' Debug monitor program for PicBasic Pro 2.2+
' Requires PICmicro with hardware serial port
' 2400 NON-INVERT (USE UART MODULE)
@ DEVICE PIC16F877,WDT_OFF,HS_OSC,LVP_OFF,BOD_OFF
DEFINE  OSC     20
DEBUG_ADDRESS   VAR     WORD BANK0 SYSTEM ' Program address
DEBUG_STACK     VAR     BYTE BANK0 SYSTEM ' Stack level
DEBUGCHAR       VAR     BYTE BANK0  ' Debug monitor command character
DEBUGFLAGS      VAR     BYTE BANK0  ' Debug monitor flags
debugi          VAR     BYTE BANK0  ' Loop counter

DEBUG_STACK = 0         ' Start stack level at 0
DEBUGFLAGS  = 0         ' Zero all debug flags
GOTO SKIPDEBUG          ' Skip around debug monitor

DEBUGMON:  
' Check for single step mode
	IF   DEBUGFLAGS.0 THEN
		HSEROUT [IHEX DEBUG_ADDRESS, $D, $A] ' Display program address
		GOTO DEBUGLOOP  ' Go to command loop
   	ENDIF

' Check for trace on
   	IF DEBUGFLAGS.1 THEN
   		HSEROUT [IHEX DEBUG_ADDRESS, $D, $A] ' Display program address
   	ENDIF

   ' Check for monitor command from serial in
   	HSERIN 0, EXITDEBUG, [DEBUGCHAR]
   	IF DEBUGCHAR != "m" THEN EXITDEBUG

' Main debug monitor loop to get command and execute
DEBUGLOOP:
' Wait for command
   	HSEROUT ["?"]           ' Display prompt
   	HSERIN  [DEBUGCHAR]     ' Get command character
   	HSEROUT [DEBUGCHAR, $D, $A] ' Echo command character to display
' Interpret command
   	LOOKDOWN DEBUGCHAR, ["dghrst"], DEBUGCHAR
   	BRANCH DEBUGCHAR, [DEBUGDUMP, DEBUGGO, DEBUGHELP, DEBUGRESET, DEBUGSTEP, DEBUGTRACE]

   	HSEROUT ["Invalid command", $D, $A]
   	GOTO DEBUGLOOP

' Dump - display stack level and register contents
DEBUGDUMP:
   	HSEROUT ["Stack level: ", #DEBUG_STACK] ' Display stack level

   	FOR DEBUGI = $0 TO $7f  ' Display all registers in bank 0
   		IF (DEBUGI // 16) = 0 THEN
   			HSEROUT [$D, $A, HEX2 DEBUGI, ":"]      ' Display address
   		ENDIF
   		PEEK  DEBUGI, DEBUGCHAR  ' Read variable
   		HSEROUT [" ", HEX2 DEBUGCHAR]   ' Display variable
	NEXT DEBUGI
	HSEROUT [$D, $A]        ' Send cr lf
   	GOTO DEBUGLOOP
' Help - display commands
DEBUGHELP:
  	HSEROUT ["Debug Monitor ver. .1", $D, $A]
   	HSEROUT ["d - display registers", $D, $A]
   	HSEROUT ["g - go", $D, $A]
   	HSEROUT ["h - help", $D, $A]
   	HSEROUT ["r - run from reset", $D, $A]
   	HSEROUT ["s - single step", $D, $A]
   	HSEROUT ["t - trace", $D, $A]
   	GOTO DEBUGLOOP

' Go - return to program execution
DEBUGGO:
   	DEBUGFLAGS.0 = 0        ' Step off
   	DEBUGFLAGS.1 = 0        ' Trace off
   	GOTO EXITDEBUG

' Reset - start program over from reset address
DEBUGRESET:
	asm
        clrf    PCLATH
        GOTO    0
  	endasm

' Step - single step one program instruction
DEBUGSTEP:
  	DEBUGFLAGS.0 = 1        ' Step on
  	DEBUGFLAGS.1 = 0        ' Trace off
  	GOTO EXITDEBUG

' Trace - run program displaying instruction addresses
DEBUGTRACE:
  	DEBUGFLAGS.0 = 0        ' Step off
  	DEBUGFLAGS.1 = 1        ' Trace on
  	GOTO EXITDEBUG

EXITDEBUG:
' End of debug monitor routine - go back to main program
  	asm
   		movf  DEBUG_ADDRESS + 1, W ; Set PCLATH with top byte of return address
        		movwf PCLATH
        		movf  DEBUG_ADDRESS, W   ; Go back to main program
     	   	movwf PCL
  	endasm

' MAIN PROGRAM 

SKIPDEBUG:
  	ON DEBUG GOTO DEBUGMON
        
MAIN:   DEBUGFLAGS.0 = 1     ' Start in debug monitor
        LOW    PORTB.3 : LOW  PORTB.4
LOOP:   ENABLE DEBUG
        HIGH   PORTB.3   		' Turn on LED connected to PORTB.3
        PAUSE  500      			' Delay 
        LOW    PORTB.3   		' Turn off LED connected to PORTB.3
        PAUSE  500      			' Delay
        DISABLE DEBUG
        HIGH    PORTB.5
        PAUSE   500
        LOW     PORTB.5
        PAUSE   500
        GOTO    LOOP     			' Go back to loop and blink LED forever        
' END 

